@@ -299,12 +299,12 @@ gequal0(GEN x)
       {
         if (gequal0(gel(x,2))) return 1;
         if (typ(gel(x,1))!=t_REAL || typ(gel(x,2))!=t_REAL) return 0;
-        return (expo(gel(x,1))>expo(gel(x,2)));
+        return (expo(gel(x,1))>=expo(gel(x,2)));
       }
       if (gequal0(gel(x,2)))
       {
         if (typ(gel(x,1))!=t_REAL || typ(gel(x,2))!=t_REAL) return 0;
-        return (expo(gel(x,2))>expo(gel(x,1)));
+        return (expo(gel(x,2))>=expo(gel(x,1)));
       }
       return 0;
 
@@ -908,7 +908,15 @@ static int
 serequal(GEN x, GEN y)
 {
   long lx;
-  if ((x[1] ^ y[1]) & (VARNBITS | SIGNBITS | VALPBITS)) return 0;
+  if (varn(x) != varn(y)) return 0;
+  if (!signe(x))
+  {
+    if (!signe(y)) return 1;
+    return valp(y) >= valp(x);
+  }
+  if (!signe(y))
+    return valp(x) >= valp(y);
+  if ((x[1] ^ y[1]) & VALPBITS) return 0;
   lx = minss(lg(x), lg(y));
   for (lx--; lx >= 2; lx--) if (!gequal(gel(x,lx), gel(y,lx))) return 0;
   return 1;
