@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -31,9 +29,9 @@ allocatemem0(GEN z)
   ulong newsize;
   if (!z) newsize = 0;
   else {
-    if (typ(z) != t_INT) pari_err(typeer,"allocatemem");
+    if (typ(z) != t_INT) pari_err_TYPE("allocatemem",z);
     newsize = itou(z);
-    if (signe(z) < 0) pari_err(talker,"negative size in allocatemem");
+    if (signe(z) < 0) pari_err_DOMAIN("allocatemem","size","<",gen_0,z);
   }
   allocatemem(newsize);
 }
