@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -100,7 +98,6 @@ static void DrawPoints(void *data, long nb, struct plot_points *p)
 static void SetForeground(void *data, long col)
 {
   Fl_Color *color = (Fl_Color*)data;
-  if (col >= numcolors) col = numcolors-1;
   fl_color(color[col]);
 }
 
@@ -135,8 +132,8 @@ void Plotter::draw()
   pl.ml = &DrawLines;
   pl.st = &DrawString;
   pl.pl = &pari_plot;
-
-  gen_rectdraw0(&pl, (void*)color, my_w, my_x, my_y, my_lw, xs, ys);
+  pl.data = (void*)color;
+  gen_rectdraw0(&pl, my_w, my_x, my_y, my_lw, xs, ys);
 }
 
 int Plotter::handle(int event)
@@ -205,7 +202,7 @@ rectdraw0(long *w, long *x, long *y, long lw)
     if (pari_daemon()) return;  // parent process returns
 
     pari_close();
-    PARI_get_plot(1);
+    PARI_get_plot();
 
     Fl::visual(FL_DOUBLE|FL_INDEX);
     win = new Plotter( w, x, y, lw);
@@ -218,10 +215,9 @@ rectdraw0(long *w, long *x, long *y, long lw)
 }
 
 void
-PARI_get_plot(long f)
+PARI_get_plot()
 /* This function initialises the structure rect.h: pari_plot */
 {
-    (void)f;
     if (pari_plot.init) return;      // pari_plot is already set
     pari_plot.width   = 400;         // width and
     pari_plot.height  = 300;         //  height of plot window
