@@ -812,6 +812,8 @@ pari_close_opts(ulong init_opts)
     if (GP_DATA->pp->cmd) free((void*)GP_DATA->pp->cmd);
     delete_dirs(GP_DATA->path);
     free((void*)GP_DATA->path->PATH);
+    delete_dirs(GP_DATA->sopath);
+    free((void*)GP_DATA->sopath->PATH);
   }
   BLOCK_SIGINT_END;
 }
