@@ -1466,7 +1466,7 @@ Fq_ellcard_SEA(GEN a4, GEN a6, GEN q, GEN T, GEN p, long smallfact)
       long t_mod_ellkt = trace_mod[1];
       if (smallfact && ell > smallfact)
       { /* does ell divide q + 1 - t ? */
-        long card_mod_ell = (umodiu(q,ell) + 1 - t_mod_ellkt) % ell ;
+        long card_mod_ell = smodss(umodiu(q,ell) + 1 - t_mod_ellkt, ell) ;
         if (!card_mod_ell)
         {
           if (DEBUGLEVEL)
@@ -1532,5 +1532,5 @@ ellsea(GEN E, GEN p, long smallfact)
   GEN a6 = modii(mulis(Rg_to_Fp(gel(E,11), p), -54), p);
   GEN card = Fp_ellcard_SEA(a4, a6, p, smallfact);
   if (!card) pari_err_PACKAGE("seadata");
-  return gerepileuptoint(av, subii(addis(p,1),card));
+  return gerepileuptoint(av, card);
 }
