@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -23,8 +21,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 
 #define NMAX 11 /* maximum degree */
 
-typedef char IND;
-typedef IND *PERM;
+typedef GEN PERM;
 typedef PERM *GROUP;
 typedef struct {
   PERM *a;
@@ -69,7 +66,7 @@ partitions_galois(long n)
     case 9: p = 30; break;
     case 10:p = 42; break;
     default:
-      if (n < 0) pari_err(talker, "partitions( %ld ) is meaningless", n);
+      if (n < 0) pari_err_TYPE("partitions_galois", stoi(n));
       av = avma; p = itos( numbpart(stoi(n)) ); avma = av; break;
   }
   T = new_chunk(p + 1); T[0] = 0;
@@ -120,13 +117,13 @@ _typ(long l,...)
 
 /* create a permutation with the N arguments of the function */
 static PERM
-_cr(long N, IND a,...)
+_cr(long N, long a,...)
 {
-  static IND x[NMAX+1];
+  static long x[NMAX+1];
   va_list args;
   long i;
 
-  va_start(args, a); x[0] = (IND)N; x[1] = a;
+  va_start(args, a); x[0] = N; x[1] = a;
   for (i=2; i<=N; i++) x[i] = va_arg(args,int);
   va_end(args); return x;
 }
@@ -135,9 +132,9 @@ static PERM
 permmul(PERM s1, PERM s2)
 {
   long i, n1 = s1[0];
-  PERM s3 = (PERM)stackmalloc((n1+1) * sizeof(IND));
-  for (i=1; i<=n1; i++) s3[i] = s1[(int)s2[i]];
-  s3[0] = (IND)n1; return s3;
+  PERM s3 = (PERM)stack_malloc((n1+1) * sizeof(long));
+  for (i=1; i<=n1; i++) s3[i] = s1[s2[i]];
+  s3[0] = n1; return s3;
 }
 
 static void
@@ -207,9 +204,8 @@ static long
 galmodp(long EVEN, GEN pol, GEN dpol, GEN TYP, long *gr, long **GR)
 {
   long i,k,l,n,nbremain;
-  byteptr d = diffptr;
   GEN p1, dtyp;
-  ulong p = 0;
+  forprime_t T;
 
   switch(degpol(pol))
   {
@@ -219,14 +215,15 @@ galmodp(long EVEN, GEN pol, GEN dpol, GEN TYP, long *gr, long **GR)
     default: nbremain = EVEN?  5:  3; break; /* case 11 */
   }
 
+  u_forprime_init(&T, 2, ULONG_MAX);
   dtyp = new_chunk(NMAX+1);
   k = gr[0]; for (i=1; i<k; i++) gr[i]=1;
   for (k=1; k<15; k++)
   {
-    NEXT_PRIME_VIADIFF_CHECK(p,d);
+    ulong p = u_forprime_next(&T);
     if (!umodiu(dpol,p)) continue; /* p divides dpol */
 
-    p1 = gel(FpX_degfact(pol,utoipos(p)),1);
+    p1 = gel(Flx_degfact(ZX_to_Flx(pol,p),p),1);
     l = lg(p1);
     dtyp[0] = evaltyp(t_VECSMALL)|evallg(l);
     for (i=1; i<l; i++) dtyp[i] = p1[l-i]; /* decreasing order */
@@ -245,7 +242,7 @@ preci(GEN o, long p)
   for (i=1; i<l; i++)
   {
     GEN x = gel(o,i);
-    if (typ(x)==t_COMPLEX) { setlg(x[1],p); setlg(x[2],p); } else setlg(x,p);
+    if (typ(x)==t_COMPLEX) { setprec(gel(x,1),p); setprec(gel(x,2),p); } else setprec(x,p);
   }
 }
 static void
@@ -254,7 +251,7 @@ fixprec(buildroot *BR)
   GEN r = BR->r;
   long i, l = lg(r), p = BR->pr;
 
-  if (p > BR->prmax) pari_err(talker,"too large precision in fixprec()");
+  if (p > BR->prmax) pari_err_BUG("fixprex [precision too large]");
   for (i = 1; i < l; i++) preci(gel(r,i), p);
 }
 
@@ -262,7 +259,7 @@ static long
 getpreci(buildroot *BR)
 {
   GEN x = gmael(BR->r,1,1);
-  return (typ(x)==t_COMPLEX)? lg(x[1]): lg(x);
+  return (typ(x)==t_COMPLEX)? realprec(gel(x,1)): realprec(x);
 }
 
 #define setcard_obj(x,n) ((x)[0] = (PERM)(n))
@@ -273,7 +270,7 @@ static PERM *
 alloc_pobj(long n, long m)
 {
   long i;
-  PERM *g = (PERM*) stackmalloc( (m+1)*sizeof(PERM) + (n+1)*m * sizeof(IND) );
+  PERM *g = (PERM*) stack_malloc( (m+1)*sizeof(PERM) + (n+1)*m * sizeof(long) );
   PERM gpt = (PERM) (g + (m+1));
 
   for (i=1; i<=m; i++) { g[i] = gpt; gpt += (n+1); }
@@ -286,7 +283,7 @@ allocgroup(long n, long card)
   GROUP gr = alloc_pobj(n,card);
   long i;
 
-  for (i=1; i<=card; i++) gr[i][0]=(char)n;
+  for (i=1; i<=card; i++) gr[i][0] = n;
   return gr;
 }
 
@@ -294,22 +291,22 @@ static pariFILE *
 galopen(const char *pre, long n, long n1, long n2)
 {
   pari_sp av = avma;
-  char *s = stackmalloc(strlen(pari_datadir) + 3 + 4 * 20 + 1 + 3);
+  char *s = stack_malloc(strlen(pari_datadir) + 3 + 4 * 20 + 1 + 3);
   pariFILE *f;
 
   (void)sprintf(s, "%s/galdata/%s%ld_%ld_%ld", pari_datadir, pre, n, n1, n2);
   f = pari_fopengz(s);
-  if (!f) pari_err(talker,"galois files not available\n[missing %s]",s);
+  if (!f) pari_err_FILE("galois file",s);
   avma = av; return f;
 }
 
 static char
 bin(char c)
 {
-  if (c>='0' && c<='9') c=c-'0';
-  else if (c>='A' && c<='Z') c=c-'A'+10;
-  else if (c>='a' && c<='z') c=c-'a'+36;
-  else pari_err(talker,"incorrect value in bin()");
+  if (c>='0' && c<='9') c -= '0';
+  else if (c>='A' && c<='Z') c -= 'A'-10;
+  else if (c>='a' && c<='z') c -= 'a'-36;
+  else pari_err_TYPE("bin [not alphanumeric]", stoi(c));
   return c;
 }
 
@@ -318,16 +315,11 @@ bin(char c)
 static void
 read_obj(PERM *g, pariFILE *f, long n, long m)
 {
-  char ch[BUFFS];
-  long i,j, k = BUFFS;
-
-  i = j = 1;
-  for(;;)
-  {
-    if (k==BUFFS) { (void)fread(ch,sizeof(char),BUFFS, f->file); k=0; }
-    g[i][j] = bin(ch[k++]);
-    if (++j>m) { j=1; if (++i>n) break; }
-  }
+  long i, j, k, N = m*n;
+  char *ch = stack_malloc(N);
+  pari_fread_chars(ch, N, f->file);
+  for (k = 0, i = 1; i <= n; i++)
+    for (j = 1; j <= m; j++,k++) g[i][j] = bin(ch[k]);
   pari_fclose(f);
 }
 #undef BUFFS
@@ -340,9 +332,9 @@ lirecoset(long n1, long n2, long n)
   char c, ch[8];
   long m, cardgr;
   pariFILE *f = galopen("COS", n, n1, n2);
-  (void)fread(&c,sizeof(char), 1, f->file); m=bin(c);
-  (void)fread(&c,sizeof(char), 1, f->file);
-  (void)fread(ch,sizeof(char), 6, f->file); cardgr=atol(ch);
+  pari_fread_chars(&c, 1, f->file); m=bin(c);
+  pari_fread_chars(&c, 1, f->file);
+  pari_fread_chars(ch, 6, f->file); cardgr=atol(ch);
   gr=allocgroup(m,cardgr);
   read_obj(gr, f,cardgr,m); return gr;
 }
@@ -353,8 +345,8 @@ lireresolv(long n1, long n2, long n, resolv *R)
   char ch[5];
   long nm, nv;
   pariFILE *f = galopen("RES", n, n1, n2);
-  (void)fread(ch,sizeof(char),5,f->file); nm = atol(ch);
-  (void)fread(ch,sizeof(char),3,f->file); nv = atol(ch);
+  pari_fread_chars(ch,5,f->file); nm = atol(ch);
+  pari_fread_chars(ch,3,f->file); nv = atol(ch);
   R->a = alloc_pobj(nv,nm);
   read_obj(R->a, f,nm,nv);
   R->nm = nm;
@@ -378,13 +370,13 @@ Monomial(GEN r, PERM bb, long nbv)
 
   for (i = 1; i <= nbv; i++)
   {
-    t = gel(r,(int)bb[i]);
-    if (typ(t) == t_COMPLEX && signe(t[1]) < 0) { s = -s; t = gneg(t); }
+    t = gel(r,bb[i]);
+    if (typ(t) == t_COMPLEX && signe(gel(t,1)) < 0) { s = -s; t = gneg(t); }
     gel(R,i) = t;
   }
   if (nbv > 2)
     gen_sort_inplace(R, (void*)&cmp_re, cmp_nodata, NULL);
-  else if (nbv == 2 && typ(R[2]) != t_COMPLEX)
+  else if (nbv == 2 && typ(gel(R,2)) != t_COMPLEX)
     swap(gel(R,1), gel(R,2));
   t = NULL;
   for (i=1; i<=nbv; i++)
@@ -395,7 +387,7 @@ Monomial(GEN r, PERM bb, long nbv)
       GEN n = gel(R,++i);
       if (!absr_cmp(gel(n,1), gel(c,1))
        && !absr_cmp(gel(n,2), gel(c,2))
-       && signe(c[2]) != signe(n[2]))
+       && signe(gel(c,2)) != signe(gel(n,2)))
         c = addrr(sqrr(gel(c,1)), sqrr(gel(c,2)));
       else
         c = gmul(c,n);
@@ -676,7 +668,7 @@ gpoly(GEN rr, long n1, long n2)
       return gsub(p1,p2);
     }
   }
-  pari_err(talker,"indefinite invariant polynomial in gpoly()");
+  pari_err_TYPE("gpoly [undefined invariant polynomial]", mkvecsmall2(n1,n2));
   return NULL; /* not reached */
 }
 
@@ -703,7 +695,7 @@ tschirn(buildroot *BR)
   long i, k, v = varn(BR->p), l = lg(BR->r);
   GEN a, h, r;
 
-  if (l >= BR->N) pari_err(bugparier,"tschirn");
+  if (l >= BR->N) pari_err_BUG("tschirn");
   if (DEBUGLEVEL)
     err_printf("\n$$$$$ Tschirnhaus transformation of degree %ld: $$$$$\n",l-1);
 
@@ -739,9 +731,9 @@ sortroots(GEN newr, GEN oldr)
         e = gexpo(gsub(gel(oldr,i), gel(newr,j)));
         if (e < e0) { e0 = e; k = j; }
       }
-    z[i] = newr[k]; t[k] = 0;
+    gel(z,i) = gel(newr,k); t[k] = 0;
   }
-  for (i=1; i<l; i++) r[i] = z[i];
+  for (i=1; i<l; i++) gel(r,i) = gel(z,i);
   return r;
 }
 
@@ -769,7 +761,7 @@ moreprec(buildroot *BR)
     BR->prmax = maxss(BR->prmax+d, (long)(BR->prmax * 1.2));
     if (DEBUGLEVEL)
       { err_printf("$$$$$ New prec = %ld\n",BR->prmax); err_flush(); }
-    ro = sortroots(cleanroots(BR->p,BR->prmax), gel(BR->r,1));
+    ro = sortroots(QX_complex_roots(BR->p,BR->prmax), gel(BR->r,1));
     delete_roots(BR);
     vectrunc_append(BR->r, gclone(ro));
     for (d = 2; d < l; d++)
@@ -786,14 +778,14 @@ get_ro(long N, GEN rr, PERM S1, PERM S2, resolv *R)
 {
   GEN r = cgetg(N+1, t_VEC);
   long i;
-  for (i=1; i<=N; i++) r[i] = rr[ (int)S1[(int)S2[i] ] ];
+  for (i=1; i<=N; i++) r[i] = rr[ S1[S2[i] ] ];
   return R->a? gpolynomial(r, R): gpoly(r,R->nm,R->nv);
 }
 /* typ(z) = t_REAL, return zi = t_INT approximation */
 static long
 sufprec_r(GEN z)
 {
-  long p = bit_accuracy( lg(z) );
+  long p = bit_prec(z);
   /* bit accuracy of fractional part large enough ? */
   return ( p - expo(z) > maxss(3*32, (long)0.2*p) );
 }
@@ -820,7 +812,7 @@ get_ro_perm(PERM S1, PERM S2, long d, resolv *R, buildroot *BR)
       if (e < -64 || sufprec(ro)) break;
       e = 0;
     }
-    BR->pr += nbits2nlong(e + 10);
+    BR->pr += nbits2extraprec(e + 10);
     moreprec(BR);
   }
   if (e > -10 || typ(roi) == t_COMPLEX) return NULL;
@@ -2203,7 +2195,7 @@ closure11(long EVEN, buildroot *BR)
     h = leafcopy(h); setvarn(h, MAXVARN);
     setvarn(r, 0); r = nffactor(h, r);
     /* S11 (P10*P10*P90) or F_110[11] (11 factors of degree 10) */
-    return (lg(r[1])-1 == 11)? 4: 8;
+    return (lgcols(r)-1 == 11)? 4: 8;
   }
 }
 
@@ -2330,7 +2322,7 @@ isin_G_H(buildroot *BR, long n1, long n2)
     for (i = 1; i < l; i++)
     {
       GEN p1 = gel(BR->r,i);
-      for (j=1; j<=N; j++) gel(z,j) = gel(p1, (int)s0[j]);
+      for (j=1; j<=N; j++) gel(z,j) = gel(p1,s0[j]);
       for (j=1; j<=N; j++) gel(p1,j) = gel(z,j);
     }
     avma = av; return n2;
@@ -2347,7 +2339,7 @@ GEN
 polgaloisnamesbig(long n, long k)
 {
   pari_sp av = avma;
-  char *s = stackmalloc(strlen(pari_datadir) + 13 + 20 + 3);
+  char *s = stack_malloc(strlen(pari_datadir) + 13 + 20 + 3);
   pariFILE *f;
   GEN V;
 
@@ -2359,12 +2351,12 @@ polgaloisnamesbig(long n, long k)
     avma = av; return strtoGENstr("");
   }
   V = gp_read_stream(f->file);
-  if (!V || typ(V)!=t_VEC || k>=lg(V))
-    pari_err(talker,"galois files %s not compatible\n",s);
+  if (!V || typ(V)!=t_VEC || k>=lg(V)) pari_err_FILE("galois file %s",s);
   pari_fclose(f);
   return gerepilecopy(av, gel(V,k));
 }
 
+/* pol a monic ZX */
 GEN
 galoisbig(GEN pol, long prec)
 {
@@ -2396,7 +2388,7 @@ galoisbig(GEN pol, long prec)
     case 9: t = galoismodulo9(EVEN,pol,dpol);  tab=tab9; break;
     case 10:t = galoismodulo10(EVEN,pol,dpol); tab=tab10; break;
     case 11:t = galoismodulo11(EVEN,pol,dpol); tab=tab11; break;
-    default: pari_err(impl,"galois in degree > 11");
+    default: pari_err_IMPL("galois in degree > 11");
       return NULL; /* not reached */
   }
   if (!t)
@@ -2411,11 +2403,11 @@ galoisbig(GEN pol, long prec)
     }
     BR.coef = z;
     BR.p = pol;
-    BR.pr = (long)(cauchy_bound(pol) / (LOG2 * BITS_IN_LONG)) + prec;
+    BR.pr = prec + nbits2extraprec((long)(cauchy_bound(pol) / LOG2));
     BR.prmax = BR.pr + BIGDEFAULTPREC-2;
     BR.N = N;
     BR.r = vectrunc_init(N+1);
-    r = gclone ( cleanroots(BR.p, BR.prmax) );
+    r = gclone ( QX_complex_roots(BR.p, BR.prmax) );
     vectrunc_append(BR.r, r); preci(r, BR.pr);
     switch(N)
     {
