@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2003  The PARI group.
+/* Copyright (C) 2003  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -219,7 +217,7 @@ group_ident_i(GEN G, GEN S)
         {
           for(i=1; t[i] != -1; i++)
             if (t[i]==s) return i;
-          pari_err(talker,"Not a group in group_ident");
+          return -1;
         }
         while (*t>=0) t++;
       }
@@ -324,7 +322,7 @@ group_ident_i(GEN G, GEN S)
             return indexgroupcentre(G,Z,good,bad)? 11: 12;
           }
         }
-        pari_err(talker,"Not a group in group_ident");
+        return -1;
       }
       while (*t!=-1) t++;
     }
@@ -427,7 +425,7 @@ group_ident_i(GEN G, GEN S)
             }
             break;
           }
-          pari_err(talker,"Not a group in group_ident");
+          return -1;
         }
         while (*t!=-1) t++;
       }
@@ -494,7 +492,7 @@ group_ident_i(GEN G, GEN S)
             }
             break;
           }
-          pari_err(talker,"Not a group in group_ident");
+          return -1;
         }
         while (*t!=-1) t++;
       }
@@ -508,9 +506,9 @@ group_ident(GEN G, GEN S)
 {
   pari_sp av = avma;
   long idx = group_ident_i(G, S);
-  avma = av;
-  if (!idx) pari_err(impl,"galoisidentify for groups of order >127");
-  return idx;
+  if (idx < 0) pari_err_TYPE("group_ident [not a group]", G);
+  if (!idx) pari_err_IMPL("galoisidentify for groups of order > 127");
+  avma = av; return idx;
 }
 
 long
@@ -540,9 +538,8 @@ group_ident_trans(GEN G, GEN S)
   long n = group_order(G), s;
   const long *t;
   if (n == 1) return 1;
-  if (n > 30)
-    pari_err(talker,
-        "Classification of transitive groups of order > 30 is not known");
+  /* N.B. known up to 32 (Cannon-Holt) */
+  if (n > 30) pari_err_IMPL("group_ident_trans [n > 30]");
   if (uisprime(n)) return 1;
   s = group_ident(G,S);
   for(t=tab;*t>=0;t++)
